from utilities import *    

def main(dataset_fobj, rules_fobj, ns_of_levels, generator, old_mem = None):
    """
    Invoke generator passing to it only the consumer_func as parameter
    Write dataset to dataset_fobj
    
    Generate a hierarchy for each attribute. The number of levels for each hierarchy is
    specified by means of ns_of_levels, which can be a number or a list of values
    (one for each attribute).
    
    Hierarchies are written on rules_fobj

    Use the ItemsMemoizer old_mem to skip dataset building and build a different hiearchy
    for the same set of items.

    Return the ItemsMemoizer.
    """

    if isinstance(ns_of_levels, int):
        ns_of_levels = itertools.cycle([ns_of_levels])

    if old_mem is None:
        mem = ItemsMemoizer()
        tx_writer = TransactionWriter(dataset_fobj)

        def tx_consumer(tx):
            if len(tx) > 0:
                return tx_writer(mem(tag_transaction(tx)))

        generator(tx_consumer)
    else:
        mem = old_mem
        
    attributes = mem.items.keys()
    attributes.sort()
    for n_of_levels, attribute in zip(ns_of_levels, attributes):
        items_lst = mem.items[attribute]
        for rule in build_hierarchy_for_attribute(items_lst, n_of_levels):
            rules_fobj.write('%s\n' % (rule,))

    return mem
