#include "tx_generator.h"
#include "gen.h"

extern Transaction *mk_tran(StringSetIter &lits, LINT tlen, Taxonomy *tax = NULL);

// Generate Transactions
//
void gen_rules(TransPar &par, PyObject *writer_func) {

  StringSet *lits;
  StringSetIter *patterns;
  Transaction *trans;
  PoissonDist *tlen;
  PyObject *list, *item, *res;
  
  LINT i = 0;
  LINT j = 0;
  
  //ofstream data_fp;
  //ofstream pat_fp;
  
  //data_fp.open(data_file, ios::trunc);
  //pat_fp.open(pat_file, ios::trunc);
  
  lits = new StringSet(par.nitems, par.lits);
  
  // Reset random seed generator for before generating transactions
  if (par.seed < 0) RandSeed::set_seed(par.seed);
  
  tlen = new PoissonDist(par.tlen-1);
  
  //  par.write(pat_fp);
  //  lits->display(pat_fp);
  
  patterns = new StringSetIter(*lits);
  for (i = 0; i < par.ntrans; i ++) {
    trans = mk_tran(*patterns, (*tlen)()+1);
    trans->sort();
    // Costruisco una lista di lunga trans.nitems, prendendo gli
    // elementi da items.
    // Chiamo writer_func passando tid e lista
    list = PyList_New(trans->nitems);
    for(j = 0; j < trans->nitems; j++) {
      item = PyInt_FromLong(trans->items[j]);
      PyList_SetItem(list, j, item);
    }

    res = PyObject_CallFunction(writer_func, "O", list);
    Py_XDECREF(list);
    Py_XDECREF(res);
    
    delete trans;
  }
}



extern "C" {


  /* #### Globals #################################### */

  /* ==== Set up the methods table ====================== */
  static PyMethodDef _tx_generator_func[] = {
    {"generate", generate, METH_VARARGS},
    {NULL, NULL}     /* Sentinel - marks the end of this structure */
  };

  /* ==== Initialize functions ====================== */
  PyMODINIT_FUNC init_tx_generator() {
    (void) Py_InitModule("_tx_generator", _tx_generator_func);
  }

  
  static PyObject* generate(PyObject* self, PyObject *args) {
    
    TransPar parameters;
    PyObject *writer_func;

    if (!PyArg_ParseTuple(args, "IfIIO", 
			  &parameters.ntrans, &parameters.tlen,
			  &parameters.nitems, &parameters.seed,
			  &writer_func)){
      return NULL;
    }

    if(NULL == writer_func || !PyCallable_Check(writer_func) ) {
      PyErr_SetString(PyExc_TypeError, "the 5-th parameter must be callable");
      return NULL;
    }

    // write cid tid ntimes [z a d f] tutti LINT
    gen_rules(parameters, writer_func);

    Py_RETURN_NONE;
  }

}

